@@ -234,8 +234,12 @@ c_pt <- function(nu)
 ## -------              --------------------------
 ## Large t (>0 or < 0)  *MUST* get a new algorithm !
 ## -------              --------------------------
-
+## >>>> ../man/pnt.Rd <<<<<<<<<
 pntR1  <- function(t, df, ncp, lower.tail = TRUE, log.p = FALSE,
+                   use.pnorm = (df > 4e5 ||
+                                ncp^2 > 2*log(2)*(-.Machine$double.min.exp)),
+                   ## /*-- 2nd part: if del > 37.6403, then p=0 below
+                   ## FIXME: test should depend on `df`, `t` AND `ncp`
                    itrmax = 1000, errmax = 1e-12, verbose = TRUE)
 {
     ## Purpose: R version of the series used in pnt() in
@@ -267,16 +271,13 @@ pntR1  <- function(t, df, ncp, lower.tail = TRUE, log.p = FALSE,
 	negdel <- TRUE ; tt <- -t; del <- -ncp
     }
 
-    if (df > 4e5 || del*del > 2*log(2)*(-.Machine$double.min.exp)) {
-        ## /*-- 2nd part: if del > 37.6403, then p=0 below
-        ## FIXME: test should depend on `df', `tt' AND `del' ! */
-        ## /* Approx. from Abramowitz & Stegun 26.7.10 (p.949) */
+    if (use.pnorm) {
+        ## Approx. from Abramowitz & Stegun 26.7.10 (p.949) -- FIXME (see above)
 	s <- 1./(4.*df)
         pnt.. <- pnorm(tt*(1 - s), del, sqrt(1. + tt*tt*2.*s),
                        lower.tail = (lower.tail != negdel), log.p=log.p)
-        cat("large 'df' or \"large\" 'ncp' -- C code would return pnorm(*) =",
+        cat("large 'df' or \"large\" 'ncp' ---> return()ing pnorm(*) =",
             format(pnt.., digits=16), "\n")
-        ### FIXME: see above !!
         return(pnt..)
     }
 
@@ -712,16 +713,22 @@ dntRwrong <- Vectorize(dntRwrong1, c("x", "df", "ncp"))
 }
 .dntJKBch <- Vectorize(.dntJKBch1, c("x", "df", "ncp"))
 
-##-FIXME: have  exp(logr(.)) in one place - *clearly* suboptimal
-## MM: really optimal in the other cases?
-logr <- function(x, a) ## == log(x / (x + a)) -- but numerically smart; x > 0, a >= 0 > -x
-    if(a < x) -log1p(a/x) else log(x / (x + a))
+## Q{MM}: is the [ a < x ] cutoff really exactly optimal?
+logr <- function(x, a) { ## == log(x / (x + a)) -- but numerically smart; x >= 0, a > -x
+    if(length(aS <- a < x) == 1L) {
+        if(aS) -log1p(a/x) else log(x / (x + a))
+    } else { # "vectors" : do ifelse(aS, .., ..) efficiently:
+        r <- a+x # of correct (recycled) length and type (numeric, mpfr,  ..)
+        r[ aS] <- -log1p((a/x)[aS])
+        r[!aS] <- log((x / r)[!aS])
+        r
+    }
+}
 
-## New "optimized" and  "mpfr-aware" version:
-dntJKBf1 <- function(x, df, ncp, log = FALSE, M = 1000)
+## New "optimized" and  "mpfr-aware" and *vectorized* (!) version:
+dntJKBf <- function(x, df, ncp, log = FALSE, M = 1000)
 {
-    stopifnot(length(x) == 1, length(df) == 1, length(ncp) == 1, length(M) == 1,
-              df >= 0, is.numeric(M), M >= 1, M == round(M))
+    stopifnot(length(M) == 1, df >= 0, is.numeric(M), M >= 1, M == round(M))
     ln2 <- log(2)
     ._1.1..M <- c(1L, seq_len(M)) # cumprod(.) = (0!, 1!, 2! ..) =  (1, 1, 2, 6, ...)
     isN <- is.numeric(x) && is.numeric(df) && is.numeric(ncp)
@@ -746,25 +753,37 @@ dntJKBf1 <- function(x, df, ncp, log = FALSE, M = 1000)
     }
 
     x2 <- x^2
-    lfac <- -ncp^2/2  - (.5*log(pi*df)+lgamma(df/2)) + logr(df, x2)*(df+1)/2
+    lfac <- -ncp^2/2 - (.5*log(pi*df)+lgamma(df/2)) + logr(df, x2)*(df+1)/2
     j <- 0:M
     lfact.j <- cumsum(log(._1.1..M)) ## == lfactorial(j)
-    dx <- ncp*x # delta * x
-    lSum <- if(dx == 0) 0 else {
-	alt <- (sign(dx) == -1) ## if(alt)  alternating sum !
-	if(ncp < 0) ncp <- -ncp
+    nd <- length(df)
+    LogRt <- 2*log(abs(ncp)) + ln2 + logr(x2, df) # (full length)
+    ## now vectorize "lSum(x,df,ncp)" :
+    lSum <- dx <- ncp*x + 0*df # delta * x  [of full length],  (correct == 0 for dx == 0)
+    if(any(negD <- dx < 0)) # if(negD[i]) "alternating sum"
+	sigPM <- rep_len(c(1,-1), length(j))
+    for(i in which(dx != 0)) { # --- compute lSum[i] ----------------
+	## use abs(ncp) : if(ncp < 0) ncp <- -ncp
 	##lterms <- lgamma((df+j + 1)/2) - lfact.j + log(x*ncp*sqrt(2)/sqrt(df+x^2))* j
-	lterms <- lgamma((df + j + 1)/2) - lfact.j + (2*log(ncp) + ln2 + logr(x2, df)) * j/2
-	if(alt) ## this is hard: even have *negative* sum {before log(.)} with mpfr,
-            ## e.g. in  dnt.1(mpfr(-4, 128), 5, 10)   ???
-            lssum(lterms, signs = rep_len(c(1,-1), length(j)), strict=FALSE)
-        else
-            lsum(lterms)
+	lterms <- lgamma((df[1L+ (i-1L)%% nd] + j + 1)/2) - lfact.j + LogRt[i] * j/2
+        lSum[i] <-
+            if(negD[i]) ## this is hard: even have *negative* sum {before log(.)} with mpfr,
+                ## e.g. in  dnt.1(mpfr(-4, 128), 5, 10)   ???
+                lssum(lterms, signs = sigPM, strict=FALSE)
+            else
+                lsum(lterms)
     }
     lf <- lfac + lSum
     if(log) lf else exp(lf)
 }
-dntJKBf <- Vectorize(dntJKBf1, c("x", "df", "ncp"))
+## No longer, as have vectorized above!
+## dntJKBf <- Vectorize(dntJKBf1, c("x", "df", "ncp"))
+## instead, from 2019-10-04 :
+dntJKBf1 <- function(x, df, ncp, log = FALSE, M = 1000) {
+    .Deprecated("dntJKBf")
+    dntJKBf(x=x, df=df, ncp=ncp, log=log, M=M)
+}
+
 
 ## Orig: ~/R/MM/NUMERICS/dpq-functions/noncentral-t-density-approx_WV.R
 ##
